/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020-2023 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package ar

import "fmt"

//-----------------------------------------------------------------------------
// Write a given subtask that is a part of a task with a given task id and that
// has a given subtask id and that been created for a package with a given name

func (a *Archive) Write(src_name string, task_id TaskId, subtask_id SubTaskId,
		subtask SubTask) {

	src_id, ok := a.package_ids[src_name]
	if !ok {
		a.counter++
		src_id = SrcId(a.counter)
		a.package_ids[src_name] = src_id
		a.packages[src_id] = src_name
		}
	subtask.Src.ID = src_id
	if a.tasks[task_id] == nil {
		a.tasks[task_id] = make(Task)
		}
	a.tasks[task_id][subtask_id] = subtask

	// Updating the time overview
	if a.time_overview[subtask.TS] == nil {
		a.time_overview[subtask.TS] = make(map[TaskId]bool)
		}
	a.time_overview[subtask.TS][task_id] = true

	// Updating the src overview
	if a.src_overview[src_id] == nil {
		a.src_overview[src_id] = make(map[TaskId]bool)
		}
	a.src_overview[src_id][task_id] = true
	}

//-----------------------------------------------------------------------------
// Update the "Ver0" and "Rel0" fields of a "SrcPackage"

const _PREV_VER_REL_ERR = "Can't write previous ver-rel"

func (a *Archive) WritePrevVerRel(updates PackagesUpdate) error {

	task, ok := a.tasks[updates.TaskId]
	if !ok {
		return fmt.Errorf(_PREV_VER_REL_ERR + " when processing task %d",
			updates.TaskId)
		}

	for _, update := range(updates.Entries) {
		src_id, ok := a.package_ids[update.SrcName]
		if !ok {
			return fmt.Errorf(_PREV_VER_REL_ERR + " of %s package " +
				"which was build in task %d", update.SrcName, updates.TaskId)
			}
		for subtask_id, subtask := range(task) {
			if subtask.Src.ID != src_id {
				continue
				}
			subtask.Src.Ver0 = update.Ver0
			subtask.Src.Rel0 = update.Rel0
			task[subtask_id] = subtask
			}
		}

	return nil
	}
