/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020-2023 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package ar

import (
	"internal/cfmt"

	"fmt"
	"sort"
	"time"
	)

//-----------------------------------------------------------------------------
// Get a slice of subtask descriptions of a task with a given id

func (a *Archive) getTaskDesc(branch string, task_id TaskId,
		src map[string]map[string]bool, format cfmt.CFormat, no_prev_ver bool) (
		datetime time.Time, desc []string, err error) {

	task, ok := a.tasks[task_id]
	if !ok {
		err = fmt.Errorf("There is no task %d in the archive", task_id)
		return
		}
	if len(task) == 0 {
		err = fmt.Errorf("Task %s is empty", task_id)
		return
		}

	// Forming a sorted slice of subtask ids
	subtask_ids := make([]SubTaskId, 0, len(task))
	for subtask_id := range task {
		subtask_ids = append(subtask_ids, subtask_id)
		}
	sort.Slice(subtask_ids, func(i, j int) bool {
		return subtask_ids[i] < subtask_ids[j]
		})

	// For every subtask id
	for _, subtask_id := range subtask_ids {

		// Getting a subtask desc, a src name and a timestamp
		subtask, ok := task[subtask_id]
		if !ok {
			err = fmt.Errorf("Can't find subtask %d", subtask_id)
			return
			}
		src_name, ok := a.packages[subtask.Src.ID]
		if !ok {
			err = fmt.Errorf("Can't find package name of subtask %d",
				subtask_id)
			return
			}
		if len(desc) == 0 {
			datetime = time.Unix(int64(subtask.TS), 0)
			}

		// The displaying of prev ver-rep is optional
		var ver_rel string
		if no_prev_ver {
			ver_rel = subtask.Src.VerRel()
		} else {
			ver_rel = fmt.Sprintf("%s >> %s",
				subtask.Src.Ver0Rel0(), subtask.Src.VerRel())
			}

		// Using standard format or a custom one
		var entry string
		if format.Size() == 0 {
			entry = fmt.Sprintf("%d:\t%s\t%s", subtask_id, src_name, ver_rel)
			if len(subtask.HashSum) > 0 {
				entry += fmt.Sprintf("\t%s:%s", subtask.BuiltFrom,
					subtask.HashSum)
				}
		} else {
			// Subtasks for packages that are not specified in the "src_name"
			// param if a custom format is specified are not displayed
			if src != nil && len(src) > 0 {
				ver_rel_set, ok := src[src_name]
				if !ok {
					continue
					}
				allowed_ver_rel := false
				for vr := range(ver_rel_set) {
					if vr == "" || vr == ver_rel {
						allowed_ver_rel = true
						break
						}
					}
				if !allowed_ver_rel {
					continue
					}
				}
			entry = format.Preface
			for _, el := range format.Elements {
				switch el.Type {
					case 'b':
						entry += fmt.Sprintf("%s", branch)
					case 'd':
						entry += fmt.Sprintf("%v", datetime)
					case 't':
						entry += fmt.Sprintf("%d", task_id)
					case 's':
						entry += fmt.Sprintf("%d", subtask_id)
					case 'n':
						entry += fmt.Sprintf("%s", src_name)
					case 'v':
						entry += fmt.Sprintf("%s", ver_rel)
					case 'h':
						entry += fmt.Sprintf("%s:%s", subtask.BuiltFrom,
							subtask.HashSum)
					default:
						err = fmt.Errorf("Invalid conversion %%v", el.Type)
						return
					}
				entry += el.Delim
				}
			}
		desc = append(desc, entry)
		}

	return
	}

//-----------------------------------------------------------------------------
// Print subtasks of a task with a given id using given format or a std one

func (a *Archive) PrintTask(branch string, task_id TaskId,
		src map[string]map[string]bool, format cfmt.CFormat,
		no_prev_ver bool) error {

	datetime, desc, err := a.getTaskDesc(branch, task_id, src, format,
		no_prev_ver)
	if err != nil {
		return err
		}

	if len(desc) < 1 {
		return nil
		}

	if format.Size() == 0 {
		fmt.Printf("Task #%d %s:\n\t/ %v /\n", task_id, branch, datetime)
		for _, entry := range desc {
			fmt.Println("\t" + entry)
			}
	} else {
		for _, entry := range desc {
			fmt.Println(entry)
			}
		}

	return nil
	}
