/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020-2023 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package ar

import (
	"fmt"
	"sort"
	"time"
	)

//-----------------------------------------------------------------------------
// Get an id of a specified src package

func (a *Archive) PackageId(src_name string) (SrcId, bool) {

	src_id, ok := a.package_ids[src_name]

	return src_id, ok
	}

//-----------------------------------------------------------------------------
// Get a map {<task_id> : <ids_of_src_packages_with_no_ver0>}, that would be
// used later as a target to read "plan/rm-src" files of these tasks and to get
// initial ver-rel of these src packages

func (a *Archive) TargetsToSearchForVer0Rel0(task_ids []TaskId) (
		map[TaskId]map[SrcId]bool) {

	targets := make(map[TaskId]map[SrcId]bool)

	for _, task_id := range task_ids {
		subtasks, ok := a.tasks[task_id]
		if !ok {
			continue
			}
		for _, subtask := range subtasks {
			if subtask.Src.Ver0 != "" {
				continue
				}
			if targets[task_id] == nil {
				targets[task_id] = make(map[SrcId]bool)
				}
			targets[task_id][subtask.Src.ID] = true
			}
		}

	return targets
	}

//-----------------------------------------------------------------------------
// Check task id

func (a *Archive) FindTaskId(id TaskId) bool {

	_, ok := a.tasks[id]

	return ok
	}

//-----------------------------------------------------------------------------
// Get ids of tasks that include build of a package with a given name

func (a *Archive) FindTasksBySrc(src map[string]map[string]bool,
		selection map[TaskId]bool) (task_ids []TaskId, err error, warn string) {

	if src == nil || len(src) < 1 {
		err = fmt.Errorf("Can't find tasks by src names because src names " +
			"aren't provided")
		return nil, err, warn
		}

	task_ids_set := make(map[TaskId]bool)

	for name, ver_rel_set := range(src) {

		// Getting an id of a src package with a given name
		src_id, ok := a.package_ids[name]
		if !ok {
			if warn != "" {
				warn += "; "
				}
			warn += fmt.Sprintf("There is no %q package", name)
			continue
			}

		// Getting a set of ids of tasks that include build of this package
		task_ids_set0, ok := a.src_overview[src_id]
		if !ok {
			err = fmt.Errorf("Src overview lacks an id of the %q package", name)
			return nil, err, warn
			}

		for ver_rel := range(ver_rel_set) {

			// Filtering the set of task ids by specified ver_rel
			if ver_rel != "" {
				for task_id := range(task_ids_set0) {
					task, ok := a.tasks[task_id]
					if !ok {
						err = fmt.Errorf("Task %d is missing in the archive " +
							"despite the fact that it's present in the src " +
							"overview", task_id)
						return nil, err, warn
						}
					for _, subtask := range(task) {
						if (subtask.Src.ID != src_id ||
								subtask.Src.VerRel() != ver_rel) {
							continue
							}
						task_ids_set[task_id] = true
						}
					}
			} else {
				for task_id := range(task_ids_set0) {
					task_ids_set[task_id] = true
					}
				}
			}
		}

	if len(task_ids_set) == 0 {
		return nil, nil, warn
		}

	// Allocating the memory for a slice of task ids
	var alloc_size int
	if selection != nil {
		alloc_size = len(selection)
	} else {
		alloc_size = len(task_ids_set)
		}
	task_ids = make([]TaskId, 0, alloc_size)

	// Adding task ids from the set to the slice
	for task_id := range task_ids_set {
		if selection != nil {
			if _, ok := selection[task_id]; !ok {
				continue
				}
			}
		task_ids = append(task_ids, task_id)
		}

	return task_ids, nil, warn
	}

//-----------------------------------------------------------------------------
// Get ids of tasks that were executed during a given time frame

func (a *Archive) FindTasksByDate(min_date, max_date time.Time,
		selection map[TaskId]bool) ([]TaskId, error) {

	// Reducing the amount of memory reallocations to a single one
	task_ids := make([]TaskId, 0, len(selection))

	// Forming a sorted slice of timestamps
	timestamps := make([]Timestamp, 0, len(a.time_overview))
	for ts := range a.time_overview {
		timestamps = append(timestamps, ts)
		}
	sort.Slice(timestamps, func(i, j int) bool {
		return timestamps[i] < timestamps[j]
		})

	// Moving forward in time
	left_condition  := min_date.IsZero()
	right_condition := max_date.IsZero()
	for _, ts := range timestamps {
		datetime := time.Unix(int64(ts), 0)
		if !left_condition {
			if !datetime.After(min_date) {
				continue
				}
			left_condition = true
			}
		if !right_condition {
			if !max_date.After(datetime) {
				break
				}
			}
		task_ids_set, ok := a.time_overview[ts]
		if !ok {
			panic("Seems like a race condition while finding task ids by date")
			}
		for task_id := range task_ids_set {
			if selection != nil {
				if _, ok := selection[task_id]; !ok {
					continue
					}
				}
			task_ids = append(task_ids, task_id)
			}
		}

	// Allocating exactly as much memory as needed if there are some task ids
	if len(task_ids) == 0 {
		return nil, nil
		}
	task_ids = append([]TaskId(nil), task_ids[:len(task_ids)]...)

	return task_ids, nil
	}
