/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020-2023 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package cfmt

import "fmt"

//-----------------------------------------------------------------------------
// Custom formatting of task descriptions

type CFormat struct {
	Preface  string
	Elements []FormatEl
	}

type FormatEl struct {
	Type  rune
	Delim string
	}

func (f * CFormat) Size() int {
	return len(f.Elements)
	}

func (f * CFormat) String() string {

	// Get the string with doubled '%' symbols
	double_prc_symbol := func(str string) string {
		var out_str string
		for _, symbol := range str {
			if symbol == '%' {
				out_str += "%%"
				continue
				}
			out_str += string(symbol)
			}
		return out_str
		}
	// - nested func

	str := double_prc_symbol(f.Preface)
	for _, el := range f.Elements {
		str += "%" + string(el.Type) + double_prc_symbol(el.Delim)
		}

	return str
	}

//-----------------------------------------------------------------------------
// Allocate new variable of a CFormat type from a given format string
// (for ex. "%d, %t, %s, %n, %v, %h")

func NewCFormat(str string) (*CFormat, error) {

	// Available conversions
	CONVERSIONS := map[rune]bool{
		'b': true, // branch name (target)
		'd': true, // date
		't': true, // task id
		's': true, // subtask id
		'n': true, // name of a package
		'v': true, // version (epoch:version-release) of a package
		'h': true, // hash sum
		}

	var f CFormat
	var flag bool
	for _, symbol := range str {
		// If current symbol follows the '%' symbol or is the '%' symbol
		if flag {
			flag = false
			if CONVERSIONS[symbol] {
				el := FormatEl{Type: symbol}
				f.Elements = append(f.Elements, el)
				continue
				}
			// Only '%' symbol could follow the '%' symbol
			// if this is not a conversion
			if symbol != '%' {
				err := fmt.Errorf("Wrong verb %%%v", symbol)
				return nil, err
				}
		} else if symbol == '%' {
			flag = true
			continue
			}
		// Current symbol is not a part of a conversion
		if f.Size() == 0 {
			f.Preface += string(symbol)
		} else {
			f.Elements[f.Size() - 1].Delim += string(symbol)
			}
		}
	// If a format string ends with the '%' symbol
	if flag {
		err := fmt.Errorf("Wrong verb %%")
		return nil, err
		}

	return &f, nil
	}
