/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020-2023 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package home

import (
	"internal/hlp"

	"io/ioutil"
	"fmt"
	"os"
	"time"
	)

// Number of hours before a dump is expired
const _EXPIRATION_PERIOD = 1

//-----------------------------------------------------------------------------
// Get a reference to a map of values of a file with a given name

func (h *Home) getValues(file_name string) map[string]string {

	for _, list_file := range h.list_files {
		if list_file.name == file_name {
			return list_file.values
			}
		}

	return nil
	}

//-----------------------------------------------------------------------------
// Create the home directory

func (h *Home) createDir() error {

	// Checking the home dir
	if f, err := os.Stat(h.abs_path); err != nil {
		if !os.IsNotExist(err) {
			return err;
			}
		// Creating the home dir
		if err = os.Mkdir(h.abs_path, 0775); err != nil {
			return err
			}
	} else {
		// If existing file is not a directory
		if !f.Mode().IsDir() {
			return fmt.Errorf("Can't create home dir because of the %q file",
				h.abs_path)
			}
		}

	return nil
	}

//-----------------------------------------------------------------------------
// Create list files

func (h *Home) createLists() error {

	for _, list_file := range h.list_files {
		err, created := h.createList(&list_file)
		if err != nil {
			return err
			}
		// Skipping the reading if file was just created
		if created {
			continue
			}
		if err := h.readList(&list_file); err != nil {
			return err
			}
		}

	return nil
	}

//-----------------------------------------------------------------------------
// Create a file with the specified name and contents in the home directory,
// return an error str and a flag indicating that the file was actually created
// (the creation will be skipped if a file with the same name already exists)

func (h *Home) createList(list_file *ListFile) (error, bool) {

	file_path := h.absPath(list_file.name)
	f, err := os.Open(file_path)
	if err != nil {
		if !os.IsNotExist(err) {
			return err, false
			}
		// Creating a list file
		if f, err = os.OpenFile(file_path, os.O_WRONLY|os.O_CREATE, 0664);
				err != nil {
			return err, false
			}
		// Constructing a CSV multiline string
		csv := hlp.ConstructCSV(list_file.values)
		// Writing the CSV multiline string into the file
		if _, err = f.WriteString(csv); err != nil {
			if another_err := f.Close(); another_err != nil {
				return fmt.Errorf("%v; %v", err, another_err), false
				}
			return err, false
			}
		return f.Close(), true
		}

	return nil, false
	}

//-----------------------------------------------------------------------------
// Read values of a list file of the home dirrectory

func (h *Home) readList(list_file *ListFile) error {

	// Reading the list file
	file_path := h.absPath(list_file.name)
	content, err := ioutil.ReadFile(file_path)
	if err != nil {
		return err
		}
	// Parsing the list
	if list_file.values == nil {
		return fmt.Errorf("Can't access values of %q list file", file_path)
		}

	return hlp.ParseCSV(string(content), list_file.values)
	}

//-----------------------------------------------------------------------------
// Check binary dumps

func (h *Home) checkDumps() error {

	h.dumps = make(map[string]bool)

	// Get a list of files of a home dir
	ls, err := ioutil.ReadDir(h.abs_path)
	if err != nil {
		return err
		}

	// Determining an acceptable time of last modification of dmp files
	time_limit := time.Now().Add((-1) * _EXPIRATION_PERIOD * time.Hour)

	// Get modification time of the "list-file"
	list_file_info, err := os.Stat(h.absPath(SRC_LIST_FILENAME))
	if err != nil {
		return err
		}
	list_mod_time := list_file_info.ModTime()

	// Going through all files of a home dir
	for _, file := range ls {
		file_name := file.Name()
		l := len(file_name)
		ext_index := l - 4
		// Skipping non dmp files
		if l < 4 || file_name[ext_index:] != ".dmp" {
			continue
			}
		// Removing this dmp file if a) it's expired and the date "before" isn't
		// less than the time of creation of this file or b) the "list-file" was
		// modified after the dmp file was created
		dump_time := file.ModTime()
		if (dump_time.Before(time_limit) &&
				(h.max_date.IsZero() || h.max_date.After(dump_time))) ||
				list_mod_time.After(dump_time) {
			if err := os.Remove(h.absPath(file.Name())); err != nil {
				return err
				}
			continue
			}
		h.dumps[file_name[:ext_index]] = true
		}

	return nil
	}
