/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020-2023 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package main
import (
	"internal/cflag"
	"internal/hlp"
	"internal/home"

	"errors"
	"flag"
	"fmt"
	"os"
	"runtime"
	"strings"
	"time"
	)

const _APP_NAME = "alt-tasks"

const _D_T_S_EVR = 0
const _RM_SRC = 1

var cpu_n = runtime.NumCPU()

//-----------------------------------------------------------------------------
// Command line arguments

var target   string
var src_name string
var task_id  uint
var no_prev_ver bool
var update   bool
var notrace  bool
var verbose  bool

const _DATE_DESC = "Date in YYYY-MM-DD format"
const _FMT_DESC  = `Custom format of an output using following conversions:
	%b - branch name (target)
	%d - date
	%t - task id
	%s - subtask id
	%n - name of a package
	%v - previous and new version (epoch:version-release) of a package,
or only a new one if "--no_prev_ver" flag is engaged
	%h - (gear|srpm):hashsum`
var after_date  = cflag.Date("after",  _DATE_DESC)
var before_date = cflag.Date("before", _DATE_DESC)
var format = cflag.Fmt("format", _FMT_DESC)

func init() {
	flag.StringVar(&target, "target", "", "Alias of a path stated in " +
		"\"" + home.SRC_LIST_FILENAME + "\" (all of them if not specified)")
	flag.StringVar(&src_name, "src_name", "", "Name of a source package " +
		"(full name <name>-<ver>-<rel>.src.rpm or just <name>), or an " +
		"absolute path to a file containing a list of src package names")
	flag.UintVar(&task_id, "id", 0, "Task Id")
	flag.BoolVar(&no_prev_ver,  "no_prev_ver",  false, "Do no display " +
		"previous version of a built package")
	flag.BoolVar(&update,  "update",  false, "Do not use existing binary dumps")
	flag.BoolVar(&notrace, "notrace", false, "Do not make binary dumps")
	flag.BoolVar(&verbose, "verbose", false, "Verbose output")
	}

//-----------------------------------------------------------------------------
// Terminate the program with a given error message

func fatalErr(err error) {
	hlp.PrintErr(err.Error())
	os.Exit(1)
	}

//-----------------------------------------------------------------------------
// Check given arguments

func checkArgs() error {

	err := ""
	got_range := !after_date.IsZero() || !before_date.IsZero()

	if len(flag.Args()) > 0 {
		unflagged := strings.Join(flag.Args(), ", ")
		err = fmt.Sprintf("Unflaged arguments %q", unflagged)
	} else if task_id != 0 {
		if src_name != "" {
			err = "Can't use 'id' and 'src_name' simultaneously"
		} else if got_range {
			err = "Can't use 'id' flag with 'src_name'/'after'/'before' flags"
			}
	} else if src_name == "" && !got_range {
		err = "There is no search criteria"
	} else if full_range := !after_date.IsZero() && !before_date.IsZero();
			full_range && after_date.After(*before_date) {
		err = "'after' date should be lesser than 'before' date"
	} else if after_date.After(time.Now()) {
		err = "'after' date should not be in the future"
		}

	if err != "" {
		return errors.New(err)
		}
	return nil
	}

//-----------------------------------------------------------------------------

// Recursively search for and parse all "d-t-s-evr.list" files in a given dir,
// form a task archive and print those tasks that satisfy given conditions

func main() {

	// Parsing given arguments (printing help if no arguments were passed)
	flag.Parse()
	if len(os.Args) == 1 {
		flag.Usage()
		os.Exit(2)
		}

	// Determining a file that holds a map of src targets
	list_file, err := home.NewListFile(home.SRC_LIST_FILENAME,
		home.SRC_LIST_SAMPLE)
	if err != nil {
		fatalErr(err)
		}

	// Recreating a home dir
	app_home, err := home.PrepareHome(_APP_NAME, []home.ListFile{*list_file,},
		*before_date)
	if err != nil {
		fatalErr(err)
		}

	// Checking the values of a given arguments
	if err := checkArgs(); err != nil {
		fatalErr(err)
		}

	// Creating a Timer obj
	timer := hlp.Timer{!verbose}

	// If target is not specified then all targets would be processed
	targets := []string{}
	if target != "" {
		targets = append(targets, target)
	} else {
		targets = app_home.Targets()
		if len(targets) < 1 {
			err := fmt.Errorf("Can't determine the targets")
			fatalErr(err)
			}
		}

	for _, target := range(targets) {

		// Creating a worker obj that will form and handle an archive of tasks
		w := NewWorker(cpu_n)

		// Using a binary dump if there is one
		dump_path, dump_exists := app_home.DumpPath(target)
		if !update && dump_exists {
			msg := fmt.Sprintf("Decoding an archive from %q", dump_path)
			task_timing := timer.StartTask(msg)
			if err := w.loadDump(dump_path); err != nil {
				hlp.PrintErr(err.Error())
			} else {
				task_timing()
				}
			}

		// Flag that shows either a new scanning was performed or not
		new_scan := false

		// Scanning the target dir if binary dump had no use
		if w.archive.Empty() {

			new_scan = true

			// Getting a src dir path
			path, err := app_home.GetVal(home.SRC_LIST_FILENAME, target)
			if err != nil {
				fatalErr(err)
				}

			msg := fmt.Sprintf("Scanning %q", path)
			task_timing := timer.StartTask(msg)
			if err := w.gatherTasksInfo(path); err != nil {
				fatalErr(err)
				}
			task_timing()
			}

		msg := "Selecting tasks that meet the given criteria"
		task_timing := timer.StartTask(msg)
		if err := w.showMatchingTasks(target, task_id, src_name, *after_date,
				*before_date, *format, no_prev_ver, verbose); err != nil {
			fatalErr(err)
			}
		task_timing()

		// Terminating if there is nothing to save or binary dump is unwanted
		if !new_scan || notrace {
			continue
			}

		msg = fmt.Sprintf("Making a binary dump into %q", dump_path)
		task_timing = timer.StartTask(msg)
		if err := w.makeDump(dump_path); err != nil {
			fatalErr(err)
			}
		task_timing()
		}
	}
