/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020-2023 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package main
import (
	"internal/ar"
	"internal/cfmt"
	"internal/hlp"
	"internal/strval"

	"fmt"
	"io/ioutil"
	"os"
	"path/filepath"
	"sort"
	"strconv"
	"strings"
	"sync"
	"time"
	)

const _PLAN_FILE_SUFFIX = "/plan/rm-src"

type worker struct {
	archive     *ar.Archive
	tasks_info  chan task_info  // for passing task records of src packages
	packages_updates chan ar.PackagesUpdate // for passing version updates that happened in course of the tasks
	semaphores  chan struct{}  // for maintaining a quota of goroutines
	parsing_wg  sync.WaitGroup // for waiting for parsers finish their jobs
	writing_wg  sync.WaitGroup // for waiting for a writer finish it's jobs
	}

type task_info struct {
	subject string
	entries []entry
	}

type entry  struct {
	task_id    ar.TaskId
	subtask_id ar.SubTaskId
	subtask    ar.SubTask
	}

func NewWorker(cpu_n int) *worker {
	var w worker
	w.archive    = ar.NewArchive()
	w.tasks_info = make(chan task_info, cpu_n)
	w.packages_updates = make(chan ar.PackagesUpdate, cpu_n)
	w.semaphores = make(chan struct{}, cpu_n)
	return &w
	}

//-----------------------------------------------------------------------------
// Receive extracted data and write it to the archive in a background

func (w *worker) startParsing(chan_id uint) error {

	w.writing_wg.Add(1)

	if chan_id == _D_T_S_EVR {
		go w.writeDocuments()
	} else if chan_id == _RM_SRC {
		go w.writePackagesUpdates()
	} else {
		return fmt.Errorf("Unknown channel id %d", chan_id)
		}

	return nil
	}

//-----------------------------------------------------------------------------
// Wait for parsers to finish their jobs and then close the channel

func (w *worker) finishParsing(chan_id uint) error {

	w.parsing_wg.Wait()
	defer w.writing_wg.Wait()

	if chan_id == _D_T_S_EVR {
		close(w.tasks_info)
	} else if chan_id == _RM_SRC {
		close(w.packages_updates)
	} else {
		return fmt.Errorf("Unknown channel id %d", chan_id)
		}

	return nil
	}

//-----------------------------------------------------------------------------
// Handle a "d-t-s-evr.list" file

func (w *worker) handleBuildList(path string, f os.FileInfo, err error) error {

	if err != nil {
		return err
		}

	// Checking if it's a searched file
	if !f.Mode().IsRegular() || filepath.Base(path) != "d-t-s-evr.list" {
		return nil
		}
	// Trying to read a file that is a searched file
	data, err := ioutil.ReadFile(path)
	if err != nil {
		return fmt.Errorf("Parsing %q, %v\n", path, err)
		}

	// Parsing this file in a new thread
	w.parsing_wg.Add(1)
	go w.parseBuildList(path, data)

	return nil
	}

//-----------------------------------------------------------------------------
// Handle a "plan/rm-src" file

func (w *worker) handlePlanFile(path string, src_ids map[ar.SrcId]bool) error {

	// Specified file may not exist
	_, err := os.Stat(path)
	if err != nil {
		return nil
		}

	// Trying to read specified file
	data, err := ioutil.ReadFile(path)
	if err != nil {
		return fmt.Errorf("Parsing %q, %v\n", path, err)
		}

	// Parsing this file in a new thread
	w.parsing_wg.Add(1)
	go w.parsePlanFile(path, data, src_ids)

	return nil
	}

//-----------------------------------------------------------------------------
// Parse a given data of a "d-t-s-evr.list" file

func (w *worker) parseBuildList(path string, data []byte) {

	// Capturing a marker
	w.semaphores <- struct{}{}

	// Decrementing the counter of running threads and releasing the marker
	// on the way out
	defer func() {
		w.parsing_wg.Done()
		<- w.semaphores
		}()

	// Getting a name of a src package
	src_name := filepath.Base(filepath.Dir(path))

	// Preparing a container
	task := task_info{src_name, make([]entry, 0, 10)}

	var ver_rel0 strval.PairDash

	// For every line of data
	NextLine:
	for n, line := range strings.Split(string(data), "\n") {

		// Breaking up a line to a number of words
		columns := strings.Fields(line)
		if len(columns) < 4 || columns[2] == "-" {
			continue
			}

		// Getting values of all the fields from the columns

		var timestamp  = new(ar.Timestamp)
		var task_id    = new(strval.Unsigned)
		var subtask_id = new(strval.Unsigned)
		var ver_rel    = new(strval.PairDash)
		var build      = new(strval.PairColon)

		// Putting various data fields into one container
		var fields = make([]strval.Assignable, 0, 5)
		fields = append(fields, timestamp, task_id, subtask_id, ver_rel)
		if len(columns) > 4 {
			fields = append(fields, build)
			}
		// Processing all fields stored in the container
		for i, p := range fields {
			if err := p.Assign(columns[i]); err != nil {
				hlp.PrintWarn(fmt.Sprintf("Format violation in file %q " +
					"on line %d: %s", path, n, err.Error()))
				continue NextLine
				}
			}
		// Forming a sub task description
		src_package := ar.SrcPackage{0, ver_rel.First, ver_rel.Second,
			ver_rel0.First, ver_rel0.Second}
		subtask := ar.SubTask{*timestamp, src_package, build.First,
			build.Second}
		task.entries = append(task.entries,
			entry{ar.TaskId(*task_id), ar.SubTaskId(*subtask_id), subtask})

		ver_rel0 = *ver_rel
		}

	w.tasks_info <- task
	}

//-----------------------------------------------------------------------------
// Parse a given data of a "...<task_id>/repo/rm-src" file

func (w *worker) parsePlanFile(path string, data []byte,
		src_ids map[ar.SrcId]bool) error {

	// Capturing a marker
	w.semaphores <- struct{}{}

	// Decrementing the counter of running threads and releasing the marker
	// on the way out
	defer func() {
		w.parsing_wg.Done()
		<- w.semaphores
		}()

	// Getting a task id
	sub_path := strings.TrimSuffix(path, _PLAN_FILE_SUFFIX)
	index := strings.LastIndex(sub_path, "/")
	task_id, err := strconv.Atoi(sub_path[(index + 1):])
	if err != nil {
		return err
		}

	// Preparing a container
	upd := ar.PackagesUpdate{ar.TaskId(task_id), make([]ar.PackageUpdate, 0, 1)}

	// For every line of data
	for _, line := range strings.Split(string(data), "\n") {

		// Breaking up a line to a number of words
		columns := strings.Fields(line)
		if len(columns) < 2 {
			continue
			}

		src_id, ok := w.archive.PackageId(columns[0])
		if !ok {
			continue
			}

		if _, ok := src_ids[src_id]; !ok {
			continue
			}

		ver_rel := new(strval.PairDash)
		ver_rel.Assign(columns[1])
		upd.Entries = append(upd.Entries,
			ar.PackageUpdate{columns[0], ver_rel.First, ver_rel.Second})
		}

	w.packages_updates <- upd

	return nil
	}

//-----------------------------------------------------------------------------
// Writing a gathered data into the archive

func (w *worker) writeDocuments() {

	// Decrementing the counter of running threads on the way out
	defer w.writing_wg.Done()
	for task := range w.tasks_info {
		for _, e := range task.entries {
			w.archive.Write(task.subject, e.task_id, e.subtask_id, e.subtask)
			}
		}
	}

//-----------------------------------------------------------------------------
// Write version updates of the packages into the archive

func (w *worker) writePackagesUpdates() {

	// Decrementing the counter of running threads on the way out
	defer w.writing_wg.Done()
	for packages_update := range w.packages_updates {
		w.archive.WritePrevVerRel(packages_update)
		}
	}

//-----------------------------------------------------------------------------
// Gather basic info of the tasks, start scanning from the specified dir

func (w *worker) gatherTasksInfo(path string) error {

	if err := w.startParsing(_D_T_S_EVR); err != nil {
		return err
		}
	if err := hlp.CheckTargetDir(path); err != nil {
		return err
		}
	if err := filepath.Walk(path, w.handleBuildList); err != nil {
		return err
		}
	w.finishParsing(_D_T_S_EVR)

	return nil
	}

//-----------------------------------------------------------------------------
// Select tasks that match a given criteria

func (w *worker) getMatchingTasks(task_id uint, src map[string]map[string]bool,
		after_date, before_date time.Time) (task_ids []ar.TaskId, err error,
		warn string) {

	var selection map[ar.TaskId]bool

	got_range := !after_date.IsZero() || !before_date.IsZero()

	if task_id != 0 {
		// Finding task with a given id
		if ok := w.archive.FindTaskId(ar.TaskId(task_id)); ok {
			task_ids = []ar.TaskId{ar.TaskId(task_id)}
			}
		return task_ids, nil, warn
		}
	if src != nil && len(src) > 0 {
		// Finding tasks for a given src name and sorting them
		task_ids, err, warn = w.archive.FindTasksBySrc(src, nil)
		if err != nil {
			return nil, err, warn
			}
		// Forming a selection set to use with other conditions
		if got_range {
			selection = make(map[ar.TaskId]bool)
			if task_ids != nil {
				for _, task_id := range task_ids {
					selection[task_id] = true
					}
				}
		} else if task_ids != nil {
			// Sorting the slice of task ids
			sort.Slice(task_ids, func(i, j int) bool {
				return task_ids[i] < task_ids[j]
				})
			}
		}
	if got_range {
		// Finding tasks for a given time frame
		task_ids, err = w.archive.FindTasksByDate(after_date, before_date,
			selection)
		if err != nil {
			return nil, err, warn
			}
		}

	return task_ids, err, warn
	}

//-----------------------------------------------------------------------------
// Print info of a tasks with given ids

func (w *worker) showTasks(branch string, task_ids []ar.TaskId,
		src map[string]map[string]bool, format cfmt.CFormat, no_prev_ver bool,
		verbose bool) {

	// If no tasks have been found
	if task_ids == nil || len(task_ids) == 0 {
		if verbose {
			fmt.Println("-- none --")
			}
		return
		}

	// Printing all the matched tasks
	for _, task_id := range task_ids {
		w.archive.PrintTask(branch, task_id, src, format, no_prev_ver)
		}
	}

//-----------------------------------------------------------------------------
// Select tasks that match a given criteria and print the selection

func (w *worker) showMatchingTasks(branch string, task_id uint, src_name string,
		after_date, before_date time.Time, format cfmt.CFormat,
		no_prev_ver bool, verbose bool) (err error) {

	src := make(map[string]map[string]bool)

	if src_name != "" {
		// If file path of a src list is given
		if src_name[0] == '/' {
			src, err = hlp.ReadSrcList(src_name)
			if err != nil {
				return err
				}
		} else {
			src[src_name] = make(map[string]bool)
			src[src_name][""] = true
			}
		}

	task_ids, err, warn := w.getMatchingTasks(task_id, src, after_date,
		before_date)
	if err != nil {
		return err
		}
	if warn != "" && verbose {
		hlp.PrintWarn(warn)
		}

	// Gathering previous versions and releases of built packages
	if !no_prev_ver {

		if err = w.startParsing(_RM_SRC); err != nil {
			return err
			}

		targets := w.archive.TargetsToSearchForVer0Rel0(task_ids)

		// Reading version updates from files
		// /tasks/archive/done/_<cluster_id>/<task_id>/plan/rm-src
		// and write them to the archive
		for task_id, src_ids := range(targets) {
			path := filepath.Join(hlp.TaskDir(uint(task_id)), _PLAN_FILE_SUFFIX)
			if err := w.handlePlanFile(path, src_ids); err != nil {
				return err
				}
			}

		w.finishParsing(_RM_SRC)
		}

	if verbose {
		hlp.PrintDelimLine()
		defer hlp.PrintDelimLine()
		}

	w.showTasks(branch, task_ids, src, format, no_prev_ver, verbose)

	return nil
	}

//-----------------------------------------------------------------------------
// Make/load a binary dump

const _NOT_INITIALIZED = "archive is not initialized"

func (w *worker) makeDump(path string) error {
	if w.archive == nil {
		return fmt.Errorf(_NOT_INITIALIZED)
		}
	// Encoding an archive
	b, err := w.archive.Encode()
	if err != nil {
		return err
		}
	// Writing the encoded archive into the file
	return ioutil.WriteFile(path, b, 0644)
	}

func (w *worker) loadDump(path string) error {
	if w.archive == nil {
		return fmt.Errorf(_NOT_INITIALIZED)
		}
	// Reading binary file
	b, err := ioutil.ReadFile(path)
	if err != nil {
		return err
		}
	// Decoding the slice of bytes of an encoded archive
	return w.archive.Decode(b)
	}
